@@ -10,10 +10,14 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use super::*;
+#[cfg(not(feature = "std"))]
+use alloc::{boxed::Box, vec::Vec};
+
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
+use crate::ast::*;
+
 /// The most complete variant of a `SELECT` query expression, optionally
 /// including `WITH`, `UNION` / other set operations, and `ORDER BY`.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
@@ -31,6 +35,8 @@ pub struct Query {
     pub offset: Option<Offset>,
     /// `FETCH { FIRST | NEXT } <N> [ PERCENT ] { ROW | ROWS } | { ONLY | WITH TIES }`
     pub fetch: Option<Fetch>,
+    /// `FOR { UPDATE | SHARE }`
+    pub lock: Option<LockType>,
 }
 
 impl fmt::Display for Query {
@@ -51,6 +57,9 @@ impl fmt::Display for Query {
         if let Some(ref fetch) = self.fetch {
             write!(f, " {}", fetch)?;
         }
+        if let Some(ref lock) = self.lock {
+            write!(f, " {}", lock)?;
+        }
         Ok(())
     }
 }
@@ -117,7 +126,7 @@ impl fmt::Display for SetOperator {
 }
 
 /// A restricted variant of `SELECT` (without CTEs/`ORDER BY`), which may
-/// appear either as the only body item of an `SQLQuery`, or as an operand
+/// appear either as the only body item of a `Query`, or as an operand
 /// to a set operation like `UNION`.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
@@ -573,6 +582,23 @@ impl fmt::Display for Fetch {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub enum LockType {
+    Share,
+    Update,
+}
+
+impl fmt::Display for LockType {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let select_lock = match self {
+            LockType::Share => "FOR SHARE",
+            LockType::Update => "FOR UPDATE",
+        };
+        write!(f, "{}", select_lock)
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Top {
